# coding=utf8

CYRILLIC = {       # instead of:
    u'ё': u'e',    # io / yo
    u'у': u'y',    # u
    u'х': u'h',    # kh
    u'щ': u'sch',  # shch
    u'ю': u'u',    # iu / yu
    u'я': u'ya',   # ia
}

GERMAN = {         # instead of:
    u'ä': u'ae',   # a
    u'ö': u'oe',   # o
    u'ü': u'ue',   # u
}

GREEK = {          # instead of:
    u'Ξ': u'X',    # Ks
    u'χ': u'ch',   # kh

    u'ϒ': u'Y',    # U
    u'υ': u'y',    # u
    u'ύ': u'y',    # ...
    u'ϋ': u'y',
    u'ΰ': u'y',
}
