<div class="control-group" title="{{ _('Host against which to check for internet connectivity')|edq }}" data-bind="css: {error: server_onlineCheckBroken, success: server_onlineCheckOk}">
    <label class="control-label" for="settings-serverOnlineCheckHost">{{ _('Host IP') }}</label>
    <div class="controls">
        <input type="text" class="input-small" data-bind="value: server_onlineCheck_host" id="settings-serverOnlineCheckHost">
    </div>
</div>
