<div class="control-group">
    <div class="controls">
        <label class="checkbox">
            <input type="checkbox" data-bind="checked: server_pluginBlacklist_enabled" id="settings-serverPluginBlacklistEnabled"> {{ _('Enable plugin blacklist processing on startup') }}
            <span class="help-block">{{ _('Any changes take effect only on the next server start.') }}</span>
        </label>
    </div>
</div>
