#settings_plugin_softwareupdate_updatelist {
  display: block;
  width: auto;
  overflow-x: hidden;
  padding-right: 2px;
  margin-bottom: 1em;
}

div.swupdate-advanced {
  margin-bottom: 5px;
}

td.settings_plugin_softwareupdate_column_update {
  width: 16px;

  a {
    text-decoration: none;
    color: #000;
  }
}

td.settings_plugin_softwareupdate_column_information {
  .line {
    display: block;
  }

  .settings_plugin_softwareupdate_release_channel {
    line-height: 20px;
    height: 20px;
    font-size: 9px;
    margin-bottom: 0;
    vertical-align: baseline;
  }
}

#settings_plugin_softwareupdate_updatelog_list {
  .line {
    display: block;
  }

  td.settings_plugin_softwareupdate_column_datetime {
    width: 150px;
  }
}

#settings_plugin_softwareupdate_workingdialog_output {
  font-size: 0.8em;

  .message {
    font-weight: bold;
  }

  .separator {
    font-weight: bold;
    color: #666666;
  }

  .stdout {
    color: #333333;
  }

  .stderr {
    color: #990000;
  }

  .call {
    color: #000099;
  }

  .message_error {
    .message;
    .stderr;
  }
}

.softwareupdate_notification {
  ul {
    margin: 10px 0 10px 25px;

    .name {
      text-overflow: ellipsis;
      white-space: nowrap;
      overflow: hidden;
      display: block;
    }
  }
}

.progress-softwareupdate.progress {
  position: relative;
  display: flex;
  overflow: hidden;
  height: 25px;
  font-size: 12px;

  .bar {
    display: flex;
    justify-content: center;
    align-items: center;
    width: 100%;
    color: white;
    height: 25px;
    font-size: 12px;
  }

  .progress-text {
    position: absolute;
    display: flex;
    justify-content: center;
    align-items: center;
    left: 0;
    right: 0;
    top: 0;
    bottom: 0;
    background: white;
    transition: clip-path 1s linear;
    height: 25px;
    font-size: 12px;
  }
}
