<div class="control-group" data-bind="visible: config_checkType() === 'github_release' || config_checkType() === 'github_commit'">
    <label class="control-label">{{ _('OctoPrint <code>pip</code> target') }}</label>
    <div class="controls">
        <input type="text" class="input-block-level" data-bind="value: config_pipTarget">
        <span class="help-block">{{ _('You should normally not have to change this.') }}</span>
    </div>
</div>
