<div class="control-group" title="{{ _('How long to cache the blacklist, in minutes. You should normally not have to change this.')|edq }}">
    <label class="control-label" for="settings-serverPluginBlacklistTtl">{{ _('Blacklist cache TTL') }}</label>
    <div class="controls">
        <div class="input-append">
            <input type="number" class="input-mini" data-bind="value: server_pluginBlacklist_ttl" id="settings-serverPluginBlacklistTtl">
            <span class="add-on">min</span>
        </div>
    </div>
</div>
