<div class="control-group" title="{{ _('Timeout after which to unload the stream if the control tab is not visible')|edq }}">
    <label class="control-label" for="settings-webcamStreamTimeout">{{ _('Stream timeout') }}</label>
    <div class="controls">
        <div class="input-append">
            <input type="number" min="1" class="input-mini text-right" data-bind="value: streamTimeout" id="settings-webcamStreamTimeout">
            <span class="add-on">sec</span>
        </div>
    </div>
</div>
