<div class="control-group">
    <div class="controls">
        <label class="checkbox">
            <input type="checkbox" data-bind="checked: webcam_timelapseEnabled" id="settings-webcamTimelapseEnabled"> {{ _('Enable timelapse support') }}
        </label>
    </div>
</div>
