<div class="control-group" title="{{ _('Aspect ratio of the webcam stream, either 16:9 or 4:3')|edq }}">
    <label class="control-label" for="settings-webcamStreamRatio">{{ _('Stream aspect ratio') }}</label>
    <div class="controls">
        <select data-bind="options: available_ratios, value: streamRatio" id="settings-webcamStreamRatio"></select>
        <span class="help-inline">{% trans %}If the stream has a different aspect ratio than configured here it will be letterboxed.{% endtrans %}</span>
    </div>
</div>
