<div class="footer">
    <ul id="footer_version" class="pull-left muted">
        <li><small>{{ _('OctoPrint') }} <span class="version">{{ display_version|e }}</span></small></li>
        <li><small>{{ _('Python') }} <span class="python_version">{{ python_version|e }}</span></small></li>
    </ul>
    <ul id="footer_links" class="pull-right">
        <li><a href="https://octoprint.org" target="_blank" rel="noreferrer noopener"><i class="fas fa-home"></i> {{ _('Web') }}</a></li>
        <li><a href="https://community.octoprint.org" target="_blank" rel="noreferrer noopener"><i class="fab fa-discourse"></i> {{ _('Forum') }}</a></li>
        <li><a href="https://discord.octoprint.org" target="_blank" rel="noreferrer noopener"><i class="fab fa-discord"></i> {{ _('Discord') }}</a></li>
        <li><a href="https://faq.octoprint.org" target="_blank" rel="noreferrer noopener"><i class="fas fa-question-circle"></i> {{ _('FAQ') }}</a></li>
        <li><a href="https://docs.octoprint.org" target="_blank" rel="noreferrer noopener"><i class="fas fa-book"></i> {{ _('Docs') }}</a></li>
        <li id="footer_about"><a href="javascript:void(0)" data-bind="click: function() { show('about_about') }"><i class="fas fa-info-circle"></i> {{ _('About') }}</a></li>
        <li id="footer_systeminfo" data-bind="visible: $root.loginState.hasPermissionKo($root.access.permissions.SYSTEM)"><a href="javascript:void(0)" data-bind="click: function() { show('about_systeminfo') }"><i class="fas fa-flag-checkered"></i> {{ _('System') }}</a></li>
    </ul>
</div>
