<h3>{{ _('Configure plugin blacklist processing') }}</h3>

<p>{% trans %}
    To protect against known severe issues with certain versions of third party plugins, OctoPrint supports the use
    of a centralized plugin version blacklist to automatically disable such plugin versions before they can interfere with
    normal operation, allowing you to uninstall or update them to a newer version.
{% endtrans %}</p>

<p>{% trans url="https://plugins.octoprint.org/blacklist/" %}
    By default, OctoPrint will use the blacklist hosted at <code>plugins.octoprint.org/blacklist.json</code> which you
    can also take a look at in a more human readable format <a href="{{ url }}" target="_blank">here</a>.
{% endtrans %}</p>

<p>{% trans %}
    Please decide whether to allow fetch and use of this centralized blacklist starting with the next server start.
    You may also change your decision at any time through Settings > Server right from within OctoPrint.
{% endtrans %}</p>

<div class="row-fluid">
    <a href="#" class="btn btn-danger span6" data-bind="click: function() { if(!setup() || decision()){disablePluginBlacklist()}}, enable: !setup() || decision(), css: {disabled: setup() && !decision()}">
        {{ _('Disable Plugin Blacklist Processing') }}
    </a>
    <a href="#" class="btn btn-primary span6" data-bind="click: function() { if(!setup() || !decision()){enablePluginBlacklist()}}, enable: !setup() || !decision(), css: {disabled: setup() && decision()}">
        {{ _('Enable Plugin Blacklist Processing') }}
    </a>
</div>

<div class="pluginblacklist_decision" style="display: none" data-bind="visible: setup()">
    <div class="text-center" style="display: none" data-bind="visible: decision()">{% trans %}
        Plugin blacklist processing is <strong class="text-success">enabled</strong>.
    {% endtrans %}</div>
    <div class="text-center" style="display: none" data-bind="visible: !decision()">{% trans %}
        Plugin blacklist processing is <strong class="text-danger">disabled</strong>.
    {% endtrans %}</div>
</div>
