<a id="navbar_show_prompt" class="pull-right" style="display: none" href="javascript:void(0)" data-bind="click: function() { showPrompt(); }, visible: active() && !visible()" title="{{ _('Message from your printer')|edq }}" aria-label="{{ _('Message from your printer')|edq }}">
    <i class="far fa-comment-dots"></i>
</a>
